     import(DoE.base,igraph,scatterplot3d)
     importFrom(utils,combn) 
     importFrom(sfsmisc,AsciiToInt,as.intBase,digitsBase)
     importFrom("grDevices", "grey", "rgb", "xyz.coords")
     importFrom("graphics", "abline", "axis", "box", "lines", "mtext",
             "par", "plot", "plot.default", "plot.new", "plot.window",
             "points", "segments", "strwidth", "symbols", "text",
             "title")
     importFrom("methods", "is")
     importFrom("stats", "aggregate", "as.formula", "coef", "contrasts",
             "contrasts<-", "formula", "model.matrix", "ppoints",
             "predict", "qnorm", "qqnorm", "rnorm", "terms", "setNames")
     importFrom("utils", "as.roman", "packageVersion", "sessionInfo")

     export("DanielPlot", "aliases", "aliasprint", "IAPlot", "MEPlot", 
            "cubePlot", "cubedraw", "cubecorners", "cubelabel",
            "FrF2","pb","FrF2Large","nrunsV","fold.design","add.center",
            "blockpick","blockpick.big","splitpick","leftadjust",
            "nruns", "nfac", "nclear.2fis",
            "clear.2fis","all.2fis.clear.catlg",
            "res","WLP","dominating",
            "CIG","CIGstatic","catlg","Yates","BsProb.design",
            "block.catlg","pb.list","compromise","gen2CIG",
            "FrF2.currentlychecked","blockgencreate","phimax",
            "Xcalc", "FF_from_X", "X_from_profile",
            "X_from_parts", "colpick", "makecatlg")

     S3method("[",catlg)
     S3method(print,catlg)
     S3method(print,aliases)

     S3method(MEPlot,default)
     S3method(MEPlot,design)
     S3method(IAPlot,default)
     S3method(IAPlot,design)
     S3method(DanielPlot,default)
     S3method(DanielPlot,design)

     S3method(nruns,catlg)
     S3method(nfac,catlg)
     S3method(res,catlg)
     S3method(nclear.2fis,catlg)
     S3method(clear.2fis,catlg)
#     S3method(all.2fis.clear,catlg)
     S3method(WLP,catlg)
     S3method(dominating,catlg)

     S3method(nruns,character)
     S3method(nfac,character)
     S3method(res,character)
     S3method(nclear.2fis,character)
     S3method(clear.2fis,character)
#     S3method(all.2fis.clear,character)
     S3method(WLP,character)
     S3method(dominating,character)

#     S3method(nruns,default)
#     S3method(nfac,default)
#     S3method(res,default)
#     S3method(nclear.2fis,default)
#     S3method(clear.2fis,default)
#     S3method(all.2fis.clear,default)
#     S3method(WLP,default)
#     S3method(dominating,default)
